import os
import signal
import asyncio
import json
import time

from typing import List
from typing import Dict
from typing import Set
from typing import Callable
from typing import Optional
from typing import Type

import aiohttp.web

from .ps2 import Ps2Keyboard

from .atx import Atx

from .msd import MassStorageError
from .msd import MassStorageDevice

from .streamer import Streamer

from .logging import get_logger


# =====
def _system_task(method: Callable) -> Callable:
    async def wrap(self: "Server") -> None:
        try:
            await method(self)
        except asyncio.CancelledError:
            pass
        except Exception:
            get_logger().exception("Unhandled exception, killing myself ...")
            os.kill(os.getpid(), signal.SIGTERM)
    return wrap


def _exceptions_as_400(msg: str, exceptions: List[Type[Exception]]) -> Callable:
    def make_wrapper(method: Callable) -> Callable:
        async def wrap(self: "Server", request: aiohttp.web.Request) -> aiohttp.web.Response:
            try:
                return (await method(self, request))
            except tuple(exceptions) as err:  # pylint: disable=catching-non-exception
                get_logger().exception(msg)
                return aiohttp.web.json_response({
                    "ok": False,
                    "result": {
                        "error": type(err).__name__,
                        "error_msg": str(err),
                    },
                }, status=400)
        return wrap
    return make_wrapper


def _json_200(result: Optional[Dict]=None) -> aiohttp.web.Response:
    return aiohttp.web.json_response({
        "ok": True,
        "result": (result or {}),
    })


class Server:  # pylint: disable=too-many-instance-attributes
    def __init__(
        self,
        keyboard: Ps2Keyboard,
        atx: Atx,
        msd: MassStorageDevice,
        streamer: Streamer,
        heartbeat: float,
        atx_state_poll: float,
        streamer_shutdown_delay: float,
        msd_chunk_size: int,
        loop: asyncio.AbstractEventLoop,
    ) -> None:

        self.__keyboard = keyboard
        self.__atx = atx
        self.__msd = msd
        self.__streamer = streamer
        self.__heartbeat = heartbeat
        self.__streamer_shutdown_delay = streamer_shutdown_delay
        self.__atx_state_poll = atx_state_poll
        self.__msd_chunk_size = msd_chunk_size
        self.__loop = loop

        self.__sockets: Set[aiohttp.web.WebSocketResponse] = set()
        self.__sockets_lock = asyncio.Lock()

        self.__system_tasks: List[asyncio.Task] = []

        self.__reset_streamer = False

    def run(self, host: str, port: int) -> None:
        self.__keyboard.start()

        app = aiohttp.web.Application(loop=self.__loop)

        app.router.add_get("/ws", self.__ws_handler)

        app.router.add_get("/atx", self.__atx_state_handler)
        app.router.add_post("/atx/click", self.__atx_click_handler)

        app.router.add_get("/msd", self.__msd_state_handler)
        app.router.add_post("/msd/connect", self.__msd_connect_handler)
        app.router.add_post("/msd/write", self.__msd_write_handler)

        app.router.add_get("/streamer", self.__streamer_state_handler)
        app.router.add_post("/streamer/reset", self.__streamer_reset_handler)

        app.on_shutdown.append(self.__on_shutdown)
        app.on_cleanup.append(self.__on_cleanup)

        self.__system_tasks.extend([
            self.__loop.create_task(self.__keyboard_watchdog()),
            self.__loop.create_task(self.__stream_controller()),
            self.__loop.create_task(self.__poll_dead_sockets()),
            self.__loop.create_task(self.__poll_atx_state()),
        ])

        aiohttp.web.run_app(app, host=host, port=port, print=self.__run_app_print)

    async def __ws_handler(self, request: aiohttp.web.Request) -> aiohttp.web.WebSocketResponse:
        ws = aiohttp.web.WebSocketResponse(heartbeat=self.__heartbeat)
        await ws.prepare(request)
        await self.__register_socket(ws)
        async for msg in ws:
            if msg.type == aiohttp.web.WSMsgType.TEXT:
                await ws.send_str(json.dumps({"msg_type": "echo", "msg": msg.data}))
            else:
                break
        return ws

    async def __atx_state_handler(self, _: aiohttp.web.Request) -> aiohttp.web.Response:
        return _json_200(self.__atx.get_state())

    @_exceptions_as_400("Click error", [RuntimeError])
    async def __atx_click_handler(self, request: aiohttp.web.Request) -> aiohttp.web.Response:
        button = request.query.get("button")
        if button == "power":
            await self.__atx.click_power()
        elif button == "power_long":
            await self.__atx.click_power_long()
        elif button == "reset":
            await self.__atx.click_reset()
        else:
            raise RuntimeError("Missing or invalid 'button=%s'" % (button))
        return _json_200({"clicked": button})

    async def __msd_state_handler(self, _: aiohttp.web.Request) -> aiohttp.web.Response:
        return _json_200(self.__msd.get_state())

    @_exceptions_as_400("Mass-storage error", [MassStorageError, RuntimeError])
    async def __msd_connect_handler(self, request: aiohttp.web.Request) -> aiohttp.web.Response:
        to = request.query.get("to")
        if to == "kvm":
            await self.__msd.connect_to_kvm()
            await self.__broadcast_event("msd_state", state="connected_to_kvm")  # type: ignore
        elif to == "server":
            await self.__msd.connect_to_pc()
            await self.__broadcast_event("msd_state", state="connected_to_server")  # type: ignore
        else:
            raise RuntimeError("Missing or invalid 'to=%s'" % (to))
        return _json_200(self.__msd.get_state())

    @_exceptions_as_400("Can't write data to mass-storage device", [MassStorageError, RuntimeError, OSError])
    async def __msd_write_handler(self, request: aiohttp.web.Request) -> aiohttp.web.Response:
        logger = get_logger(0)
        reader = await request.multipart()
        writed = 0
        try:
            field = await reader.next()
            if field.name != "image_name":
                raise RuntimeError("Missing 'image_name' field")
            image_name = (await field.read()).decode("utf-8")[:256]

            field = await reader.next()
            if field.name != "image_data":
                raise RuntimeError("Missing 'image_data' field")

            async with self.__msd:
                await self.__broadcast_event("msd_state", state="busy")  # type: ignore
                logger.info("Writing image %r to mass-storage device ...", image_name)
                await self.__msd.write_image_name(image_name)
                while True:
                    chunk = await field.read_chunk(self.__msd_chunk_size)
                    if not chunk:
                        break
                    writed = await self.__msd.write_image_chunk(chunk)
            await self.__broadcast_event("msd_state", state="free")  # type: ignore
        finally:
            if writed != 0:
                logger.info("Writed %d bytes to mass-storage device", writed)
        return _json_200({"writed": writed})

    async def __streamer_state_handler(self, _: aiohttp.web.Request) -> aiohttp.web.Response:
        return _json_200(self.__streamer.get_state())

    async def __streamer_reset_handler(self, _: aiohttp.web.Request) -> aiohttp.web.Response:
        self.__reset_streamer = True
        return _json_200()

    def __run_app_print(self, text: str) -> None:
        logger = get_logger()
        for line in text.strip().splitlines():
            logger.info(line.strip())

    async def __on_shutdown(self, _: aiohttp.web.Application) -> None:
        logger = get_logger(0)

        logger.info("Cancelling system tasks ...")
        for task in self.__system_tasks:
            task.cancel()
        await asyncio.gather(*self.__system_tasks)

        logger.info("Disconnecting clients ...")
        for ws in list(self.__sockets):
            await self.__remove_socket(ws)

    async def __on_cleanup(self, _: aiohttp.web.Application) -> None:
        self.__keyboard.cleanup()
        await self.__streamer.cleanup()
        await self.__msd.cleanup()

    @_system_task
    async def __keyboard_watchdog(self) -> None:
        while self.__keyboard.is_alive():
            await asyncio.sleep(0.1)
        raise RuntimeError("Keyboard dead")

    @_system_task
    async def __stream_controller(self) -> None:
        prev = 0
        shutdown_at = 0.0

        while True:
            cur = len(self.__sockets)
            if prev == 0 and cur > 0:
                if not self.__streamer.is_running():
                    await self.__streamer.start()
            elif prev > 0 and cur == 0:
                shutdown_at = time.time() + self.__streamer_shutdown_delay
            elif prev == 0 and cur == 0 and time.time() > shutdown_at:
                if self.__streamer.is_running():
                    await self.__streamer.stop()

            if self.__reset_streamer:
                if self.__streamer.is_running():
                    await self.__streamer.stop()
                    await self.__streamer.start()
                self.__reset_streamer = False

            prev = cur
            await asyncio.sleep(0.1)

    @_system_task
    async def __poll_dead_sockets(self) -> None:
        while True:
            for ws in list(self.__sockets):
                if ws.closed or not ws._req.transport:  # pylint: disable=protected-access
                    await self.__remove_socket(ws)
            await asyncio.sleep(0.1)

    @_system_task
    async def __poll_atx_state(self) -> None:
        while True:
            if self.__sockets:
                await self.__broadcast_event("atx_state", **self.__atx.get_state())
            await asyncio.sleep(self.__atx_state_poll)

    async def __broadcast_event(self, event: str, **kwargs: Dict) -> None:
        await asyncio.gather(*[
            ws.send_str(json.dumps({
                "msg_type": "event",
                "msg": {
                    "event": event,
                    "event_attrs": kwargs,
                },
            }))
            for ws in list(self.__sockets)
            if not ws.closed and ws._req.transport  # pylint: disable=protected-access
        ], return_exceptions=True)

    async def __register_socket(self, ws: aiohttp.web.WebSocketResponse) -> None:
        async with self.__sockets_lock:
            self.__sockets.add(ws)
            get_logger().info("Registered new client socket: remote=%s; id=%d; active=%d",
                              ws._req.remote, id(ws), len(self.__sockets))  # pylint: disable=protected-access

    async def __remove_socket(self, ws: aiohttp.web.WebSocketResponse) -> None:
        async with self.__sockets_lock:
            try:
                self.__sockets.remove(ws)
                get_logger().info("Removed client socket: remote=%s; id=%d; active=%d",
                                  ws._req.remote, id(ws), len(self.__sockets))  # pylint: disable=protected-access
                await ws.close()
            except Exception:
                pass



import asyncio
import asyncio.subprocess

from typing import List
from typing import Dict
from typing import Optional

from .logging import get_logger

from . import gpio


# =====
class Streamer:  # pylint: disable=too-many-instance-attributes
    def __init__(
        self,
        cap_power: int,
        conv_power: int,
        sync_delay: float,
        init_delay: float,
        width: int,
        height: int,
        cmd: List[str],
        loop: asyncio.AbstractEventLoop,
    ) -> None:

        self.__cap_power = (gpio.set_output(cap_power) if cap_power > 0 else cap_power)
        self.__conv_power = (gpio.set_output(conv_power) if conv_power > 0 else conv_power)
        self.__sync_delay = sync_delay
        self.__init_delay = init_delay
        self.__width = width
        self.__height = height
        self.__cmd = cmd

        self.__loop = loop

        self.__proc_task: Optional[asyncio.Task] = None

    async def start(self) -> None:
        assert not self.__proc_task
        get_logger().info("Starting streamer ...")
        await self.__set_hw_enabled(True)
        self.__proc_task = self.__loop.create_task(self.__process())

    async def stop(self) -> None:
        assert self.__proc_task
        get_logger().info("Stopping streamer ...")
        self.__proc_task.cancel()
        await asyncio.gather(self.__proc_task, return_exceptions=True)
        await self.__set_hw_enabled(False)
        self.__proc_task = None

    def is_running(self) -> bool:
        return bool(self.__proc_task)

    def get_state(self) -> Dict:
        return {
            "is_running": self.is_running(),
            "size": {
                "width": self.__width,
                "height": self.__height,
            },
        }

    async def cleanup(self) -> None:
        if self.is_running():
            await self.stop()

    async def __set_hw_enabled(self, enabled: bool) -> None:
        # XXX: This sequence is very important to enable converter and cap board
        if self.__cap_power > 0:
            gpio.write(self.__cap_power, enabled)
        if self.__conv_power > 0:
            if enabled:
                await asyncio.sleep(self.__sync_delay)
            gpio.write(self.__conv_power, enabled)
        if enabled:
            await asyncio.sleep(self.__init_delay)

    async def __process(self) -> None:  # pylint: disable=too-many-branches
        logger = get_logger(0)

        while True:  # pylint: disable=too-many-nested-blocks
            proc: Optional[asyncio.subprocess.Process] = None  # pylint: disable=no-member
            try:
                cmd = [part.format(width=self.__width, height=self.__height) for part in self.__cmd]
                proc = await asyncio.create_subprocess_exec(
                    *cmd,
                    stdout=asyncio.subprocess.PIPE,
                    stderr=asyncio.subprocess.STDOUT,
                )
                logger.info("Started streamer pid=%d: %s", proc.pid, cmd)

                empty = 0
                while proc.returncode is None:
                    line = (await proc.stdout.readline()).decode(errors="ignore").strip()
                    if line:
                        logger.info("streamer: %s", line)
                        empty = 0
                    else:
                        empty += 1
                        if empty == 100:  # asyncio bug
                            break

                raise RuntimeError("WTF")

            except asyncio.CancelledError:
                break

            except Exception as err:
                if proc:
                    logger.exception("Unexpected streamer error: pid=%d", proc.pid)
                else:
                    logger.exception("Can't start streamer: %s", err)
                await asyncio.sleep(1)

            finally:
                if proc and proc.returncode is None:
                    await self.__kill(proc)

    async def __kill(self, proc: asyncio.subprocess.Process) -> None:  # pylint: disable=no-member
        try:
            proc.terminate()
            await asyncio.sleep(1)
            if proc.returncode is None:
                try:
                    proc.kill()
                except Exception:
                    if proc.returncode is not None:
                        raise
            await proc.wait()
            get_logger().info("Streamer killed: pid=%d; retcode=%d", proc.pid, proc.returncode)
        except Exception:
            if proc.returncode is None:
                get_logger().exception("Can't kill streamer pid=%d", proc.pid)
            else:
                get_logger().info("Streamer killed: pid=%d; retcode=%d", proc.pid, proc.returncode)



#!/usr/bin/env python3


import sys
import signal
import asyncio
import argparse
import time

import aiohttp


# =====
async def _run_client(loop: asyncio.AbstractEventLoop, url: str) -> None:
    def stdin_callback() -> None:
        line = sys.stdin.buffer.readline().decode()
        if line:
            asyncio.ensure_future(ws.send_str(line), loop=loop)
        else:
            loop.stop()

    loop.add_reader(sys.stdin.fileno(), stdin_callback)

    async def dispatch() -> None:
        while True:
            msg = await ws.receive()
            if msg.type == aiohttp.WSMsgType.TEXT:
                print("[%.5f] Received: %s" % (time.time(), msg.data.strip()))
            else:
                if msg.type == aiohttp.WSMsgType.CLOSE:
                    await ws.close()
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    print("[%.5f] Error during receive: %s" % (time.time(), ws.exception()))
                elif msg.type == aiohttp.WSMsgType.CLOSED:
                    pass
                break

    async with aiohttp.ClientSession().ws_connect(url) as ws:
        await dispatch()


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--url", default="http://localhost:8081/ws")
    options = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.add_signal_handler(signal.SIGINT, loop.stop)
    loop.create_task(_run_client(loop, options.url))
    loop.run_forever()



import os
import struct
import asyncio
import types

from typing import Dict
from typing import NamedTuple
from typing import Callable
from typing import Type
from typing import Optional
from typing import Any

import pyudev

import aiofiles
import aiofiles.base

from .logging import get_logger


# =====
class MassStorageError(Exception):
    pass


class IsNotOperationalError(MassStorageError):
    def __init__(self) -> None:
        super().__init__("Missing path for mass-storage device")


class AlreadyConnectedToPcError(MassStorageError):
    def __init__(self) -> None:
        super().__init__("Mass-storage is already connected to Server")


class AlreadyConnectedToKvmError(MassStorageError):
    def __init__(self) -> None:
        super().__init__("Mass-storage is already connected to KVM")


class IsNotConnectedToKvmError(MassStorageError):
    def __init__(self) -> None:
        super().__init__("Mass-storage is not connected to KVM")


class IsBusyError(MassStorageError):
    def __init__(self) -> None:
        super().__init__("Mass-storage is busy (write in progress)")


class MassStorageDeviceInfo(NamedTuple):
    path: str
    size: int
    manufacturer: str
    product: str
    serial: str
    image_name: str


_DISK_META_SIZE = 4096
_DISK_META_MAGIC_SIZE = 16
_DISK_META_IMAGE_NAME_SIZE = 256
_DISK_META_PADS_SIZE = _DISK_META_SIZE - _DISK_META_IMAGE_NAME_SIZE - _DISK_META_MAGIC_SIZE * 8
_DISK_META_FORMAT = ">%dL%dc%dx%dL" % (
    _DISK_META_MAGIC_SIZE,
    _DISK_META_IMAGE_NAME_SIZE,
    _DISK_META_PADS_SIZE,
    _DISK_META_MAGIC_SIZE,
)
_DISK_META_MAGIC = [0x1ACE1ACE] * _DISK_META_MAGIC_SIZE


def _make_disk_meta(image_name: str) -> bytes:
    return struct.pack(
        _DISK_META_FORMAT,
        *_DISK_META_MAGIC,
        *memoryview((  # type: ignore
            image_name.encode("utf-8")
            + b"\x00" * _DISK_META_IMAGE_NAME_SIZE
        )[:_DISK_META_IMAGE_NAME_SIZE]).cast("c"),
        *_DISK_META_MAGIC,
    )


def _parse_disk_meta(data: bytes) -> Dict:
    disk_meta = {"image_name": ""}
    try:
        parsed = list(struct.unpack(_DISK_META_FORMAT, data))
    except struct.error:
        pass
    else:
        magic_begin = parsed[:_DISK_META_MAGIC_SIZE]
        magic_end = parsed[-_DISK_META_MAGIC_SIZE:]
        if magic_begin == magic_end == _DISK_META_MAGIC:
            image_name_bytes = b"".join(parsed[_DISK_META_MAGIC_SIZE:_DISK_META_MAGIC_SIZE + _DISK_META_IMAGE_NAME_SIZE])
            disk_meta["image_name"] = image_name_bytes.decode("utf-8", errors="ignore").strip("\x00").strip()
    return disk_meta


def explore_device(path: str) -> Optional[MassStorageDeviceInfo]:
    # udevadm info -a -p  $(udevadm info -q path -n /dev/sda)
    ctx = pyudev.Context()

    block_device = pyudev.Devices.from_device_file(ctx, path)
    try:
        size = block_device.attributes.asint("size") * 512
    except KeyError:
        return None

    usb_device = block_device.find_parent("usb", "usb_device")
    if not usb_device:
        return None

    with open(path, "rb") as device_file:
        device_file.seek(size - _DISK_META_SIZE)
        disk_meta = _parse_disk_meta(device_file.read())

    return MassStorageDeviceInfo(
        path=path,
        size=size,
        manufacturer=usb_device.attributes.asstring("manufacturer").strip(),
        product=usb_device.attributes.asstring("product").strip(),
        serial=usb_device.attributes.asstring("serial").strip(),
        image_name=disk_meta["image_name"],
    )


def _operated_and_locked(method: Callable) -> Callable:
    async def wrap(self: "MassStorageDevice", *args: Any, **kwargs: Any) -> Any:
        if self._device_file:  # pylint: disable=protected-access
            raise IsBusyError()
        if not self._device_path:  # pylint: disable=protected-access
            IsNotOperationalError()
        async with self._lock:  # pylint: disable=protected-access
            return (await method(self, *args, **kwargs))
    return wrap


class MassStorageDevice:  # pylint: disable=too-many-instance-attributes
    def __init__(
        self,
        device_path: str,
        init_delay: float,
        write_meta: bool,
        loop: asyncio.AbstractEventLoop,
    ) -> None:

        self._device_path = device_path
        self.__init_delay = init_delay
        self.__write_meta = write_meta
        self.__loop = loop

        self.__device_info: Optional[MassStorageDeviceInfo] = None
        self._lock = asyncio.Lock()
        self._device_file: Optional[aiofiles.base.AiofilesContextManager] = None
        self.__writed = 0

        logger = get_logger(0)
        if self._device_path:
            logger.info("Using %r as mass-storage device", self._device_path)
            try:
                logger.info("Enabled metadata writing")
                loop.run_until_complete(self.connect_to_kvm(no_delay=True))
            except Exception as err:
                if isinstance(err, MassStorageError):
                    log = logger.error
                else:
                    log = logger.exception
                log("Mass-storage device is not operational: %s", err)
                self._device_path = ""
        else:
            logger.warning("Mass-storage device is not operational")

    @_operated_and_locked
    async def connect_to_kvm(self, no_delay: bool=False) -> None:
        if self.__device_info:
            raise AlreadyConnectedToKvmError()
        # TODO: disable gpio
        if not no_delay:
            await asyncio.sleep(self.__init_delay)
        await self.__reread_device_info()
        get_logger().info("Mass-storage device switched to KVM: %s", self.__device_info)

    @_operated_and_locked
    async def connect_to_pc(self) -> None:
        if not self.__device_info:
            raise AlreadyConnectedToPcError()
        # TODO: enable gpio
        self.__device_info = None
        get_logger().info("Mass-storage device switched to Server")

    def get_state(self) -> Dict:
        return {
            "in_operate": bool(self._device_path),
            "connected_to": ("kvm" if self.__device_info else "server"),
            "is_busy": bool(self._device_file),
            "writed": self.__writed,
            "info": (self.__device_info._asdict() if self.__device_info else None),
        }

    async def cleanup(self) -> None:
        async with self._lock:
            await self.__close_device_file()
            # TODO: disable gpio

    @_operated_and_locked
    async def __aenter__(self) -> "MassStorageDevice":
        if not self.__device_info:
            raise IsNotConnectedToKvmError()
        self._device_file = await aiofiles.open(self.__device_info.path, mode="wb", buffering=0)
        self.__writed = 0
        return self

    async def write_image_name(self, image_name: str) -> None:
        async with self._lock:
            assert self._device_file
            assert self.__device_info
            if self.__write_meta:
                await self._device_file.seek(self.__device_info.size - _DISK_META_SIZE)
                await self._device_file.write(_make_disk_meta(image_name))
                await self._device_file.flush()
                await self.__loop.run_in_executor(None, os.fsync, self._device_file.fileno())
                await self._device_file.seek(0)
                await self.__reread_device_info()

    async def write_image_chunk(self, chunk: bytes) -> int:
        async with self._lock:
            assert self._device_file
            size = len(chunk)
            await self._device_file.write(chunk)
            await self._device_file.flush()
            await self.__loop.run_in_executor(None, os.fsync, self._device_file.fileno())
            self.__writed += size
            return self.__writed

    async def __aexit__(
        self,
        _exc_type: Type[BaseException],
        _exc: BaseException,
        _tb: types.TracebackType,
    ) -> None:
        async with self._lock:
            await self.__close_device_file()

    async def __reread_device_info(self) -> None:
        device_info = await self.__loop.run_in_executor(None, explore_device, self._device_path)
        if not device_info:
            raise MassStorageError("Can't explore device %r" % (self._device_path))
        self.__device_info = device_info

    async def __close_device_file(self) -> None:
        try:
            if self._device_file:
                get_logger().info("Closing mass-storage device file ...")
                await self._device_file.close()
        except Exception:
            get_logger().exception("Can't close mass-storage device file")
            # TODO: reset device file
        self._device_file = None
        self.__writed = 0



import os
import subprocess
import time

from ...application import init
from ...logging import get_logger

from ... import gpio


# =====
def main() -> None:
    config = init()
    logger = get_logger(0)

    logger.info("Cleaning up ...")
    with gpio.bcm():
        for (name, pin) in [
            ("atx_power_switch", config["atx"]["pinout"]["power_switch"]),
            ("atx_reset_switch", config["atx"]["pinout"]["reset_switch"]),
            ("streamer_cap", config["streamer"]["pinout"]["cap"]),
            ("streamer_conv", config["streamer"]["pinout"]["conv"]),
        ]:
            if pin > 0:
                logger.info("Writing value=0 to pin=%d (%s)", pin, name)
                gpio.set_output(pin, initial=False)

    streamer = os.path.basename(config["streamer"]["cmd"][0])
    logger.info("Trying to find and kill %r ...", streamer)
    try:
        subprocess.check_output(["killall", streamer], stderr=subprocess.STDOUT)
        time.sleep(3)
        subprocess.check_output(["killall", "-9", streamer], stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError:
        pass

    logger.info("Bye-bye")



#!/usr/bin/env python3


from setuptools import setup


# =====
def main() -> None:
    with open("requirements.txt") as requirements_file:
        install_requires = list(filter(None, requirements_file.read().splitlines()))

    setup(
        name="kvmd",
        version="0.10",
        url="https://github.com/mdevaev/pi-kvm",
        license="GPLv3",
        author="Maxim Devaev",
        author_email="mdevaev@gmail.com",
        description="The main Pi-KVM daemon",
        platforms="any",

        packages=[
            "kvmd",
            "kvmd.extras",
            "kvmd.extras.cleanup",
            "kvmd.extras.wscli",
        ],

        entry_points={
            "console_scripts": [
                "kvmd = kvmd:main",
                "kvmd-cleanup = kvmd.extras.cleanup:main",
                "kvmd-wscli = kvmd.extras.wscli:main",
            ],
        },

        install_requires=install_requires,

        classifiers=[
            "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
            "Development Status :: 3 - Alpha",
            "Programming Language :: Python :: 3.6",
            "Topic :: System :: Systems Administration",
            "Operating System :: POSIX :: Linux",
            "Intended Audience :: System Administrators",
            "Intended Audience :: End Users/Desktop",
            "Intended Audience :: Telecommunications Industry",
        ],
    )


if __name__ == "__main__":
    main()



import asyncio

from .application import init
from .logging import get_logger

from .ps2 import Ps2Keyboard
from .atx import Atx
from .msd import MassStorageDevice
from .streamer import Streamer
from .server import Server

from . import gpio


# =====
def main() -> None:
    config = init()
    with gpio.bcm():
        loop = asyncio.get_event_loop()

        keyboard = Ps2Keyboard(
            clock=int(config["keyboard"]["pinout"]["clock"]),
            data=int(config["keyboard"]["pinout"]["data"]),
            pulse=float(config["keyboard"]["pulse"]),
        )

        atx = Atx(
            power_led=int(config["atx"]["pinout"]["power_led"]),
            hdd_led=int(config["atx"]["pinout"]["hdd_led"]),
            power_switch=int(config["atx"]["pinout"]["power_switch"]),
            reset_switch=int(config["atx"]["pinout"]["reset_switch"]),
            click_delay=float(config["atx"]["click_delay"]),
            long_click_delay=float(config["atx"]["long_click_delay"]),
        )

        msd = MassStorageDevice(
            device_path=str(config["msd"]["device"]),
            init_delay=float(config["msd"]["init_delay"]),
            write_meta=bool(config["msd"]["write_meta"]),
            loop=loop,
        )

        streamer = Streamer(
            cap_power=int(config["streamer"]["pinout"]["cap"]),
            conv_power=int(config["streamer"]["pinout"]["conv"]),
            sync_delay=float(config["streamer"]["sync_delay"]),
            init_delay=float(config["streamer"]["init_delay"]),
            width=int(config["streamer"]["size"]["width"]),
            height=int(config["streamer"]["size"]["height"]),
            cmd=list(map(str, config["streamer"]["cmd"])),
            loop=loop,
        )

        Server(
            keyboard=keyboard,
            atx=atx,
            msd=msd,
            streamer=streamer,
            heartbeat=float(config["server"]["heartbeat"]),
            atx_state_poll=float(config["atx"]["state_poll"]),
            streamer_shutdown_delay=float(config["streamer"]["shutdown_delay"]),
            msd_chunk_size=int(config["msd"]["chunk_size"]),
            loop=loop,
        ).run(
            host=str(config["server"]["host"]),
            port=int(config["server"]["port"]),
        )

    get_logger().info("Bye-bye")
